// Copyright 10/15/98 Sun Microsystems, Inc. All Rights Reserved.
//
#pragma ident  "@(#)event_send3.cc	1.11 98/10/15 Sun Microsystems"
//
// Purpose: Send an event notification, with time in ASN.1 notation, to MIS.
//
// Usage: event_send3 [ -e <event> [ -i <event_info> ] ]
//
// Demonstrates:
//    Usage of send_event(DU,Asn1Value)
//    Encoding/decoding of Asn1Value
//
// Note:    The syntax of <event_info> is based on the event doc,
//          as found in the /opt/SUNWconn/em/etc/gdmo/ directory.
//          For example, in the dmi.gdmo doc file, you can
//          search for "communicationsAlarm" to see that syntax.
//
// Examples: 
//
//	%event_send3
//
//	%event_send3 -e 'communicationsAlarm' \
	 -i '{ probableCause localValue : 66, perceivedSeverity critical }'
//

#include <netdb.h>
#include <sys/systeminfo.h>

#include <hi.hh>
#include <hi_d3.hh>


int 
main(
	int argc, 
	char **argv
    )
{
	char event[300]       = "communicationsAlarm";
	char event_info[300]  = "{ probableCause localValue : 66, "
		                "perceivedSeverity critical }";

	// Parse the cmd line for options.

        int c;
	if (argc == 5) {
            while ((c = getopt(argc, argv, "e:i:")) != EOF) {
                switch (c) {
                  case 'e':
                        strcpy(event, optarg);
                        break;
 		  case 'i':
                        strcpy(event_info, optarg);
                        break;

                  case '?':
                        exit(1);
                }
            }
	} else if (argc == 1) {
	    cout << "No command option using the default value. " << endl;
	    cout << "Default event is " << event << endl;
	    cout << "        event_info is " << event_info << endl;
	} else {
	    cout << "Usage:  event_send3 -e <event> -i <event_info>" << endl;
	    exit(2);
	}

	
        Platform plat = Platform(duEM);
	if (plat.get_error_type()!=PMI_SUCCESS) {
		cout << plat.get_error_string() << endl;
		exit(3); 
	}
 
        // Get the host name.

        char *host = getenv("EM_SERVER");
        if (!host) {
                host = new char[MAXHOSTNAMELEN+1];
                sysinfo(SI_HOSTNAME, host, 255);
        }
 
        cout << "Connecting to ... " << host << endl;
        if (!plat.connect(host, "em_sample")) {
                cout << "Failed to connect to " << host << endl;
                cout << plat.get_error_string() << endl;
                exit(4);
        }
	cout << "Connected." << endl << endl;


	// Alarm can be generated from any object.
	// Both name and object class are required for sending event.
	
 	Image im("/systemId=\"vidhi\"/logId=string:\"AlarmLog\"","log");
	if (im.get_error_type() != PMI_SUCCESS) {
		cout << im.get_error_string() << endl;
		exit(5);
	}

	// Get the syntax for event.

	//Syntax syn = WhenList::syntax(event,plat);
	//Equivalent to Platform.get_when_syntax(event)

	Syntax syn = plat.get_when_syntax(event);
	if(!syn) {
		cout << syn.get_error_string() << endl;
		exit(6);
	}

	// Parse info using morf with correct syntax syn. Put result into mf.

	Morf mf(syn,event_info);
	if(!mf || mf.get_error_type() != PMI_SUCCESS) {
		cout << mf.get_error_string() << endl;
		exit(7);
	}

	// From mf, get the encoded value.

 	Asn1Value event_info_asn1 = mf.get_value();
	if(!mf || mf.get_error_type() != PMI_SUCCESS) {
		cout << mf.get_error_string() << endl;
		exit(8);
	}

	// Send event notification.
	// (This call can also take a third argument, Asn1Value &tm)

	if(!im.send_event(event, event_info_asn1)) { // Put the message in a queue.
		cout<< im.get_error_string() << endl;
		exit(9);
	}

	dispatch_recursive(TRUE);  // Flush message from queue to MIS.
	exit(0);
}
